<?php

/**
 * @file
 * Drush integration for Monster Menus.
 */

/**
 * Implements hook_drush_command().
 */
function monster_menus_drush_command() {
  $items['mm-check-orphan-nodes'] = array(
    'description' => t('Check for nodes not assigned to any MM page. See also mm-fix-orphan-nodes'),
    'aliases' => array('mm-orphans', 'mmcon'),
  );
  $items['mm-check-sort'] = array(
    'description' => t('Check for problems in the way menu entries are sorted. See also mm-fix-sort.'),
    'aliases' => array('mm-check', 'mmcs'),
  );
  $items['mm-create-homepage'] = array(
    'description' => t('Create a user homepage or restore an existing one.'),
    'aliases' => array('mmch'),
    'arguments' => array(
      'account' => 'The name or UID of the user to create the homepage for',
    ),
    'required-arguments' => TRUE,
  );
  $items['mm-disable-homepage'] = array(
    'description' => t('Move a user homepage to the disabled section.'),
    'aliases' => array('mmdh'),
    'arguments' => array(
      'account' => 'The name or UID of the user to disable',
    ),
    'required-arguments' => TRUE,
  );
  $items['mm-dump'] = array(
    'description' => t('Dump a section of the tree as CSV.'),
    'aliases' => array('mmd'),
    'arguments' => array(
      'mmtid' => 'The Tree ID of the page to start at',
    ),
    'required-arguments' => TRUE,
  );
  $items['mm-export'] = array(
    'description' => t('Export a section of the tree in a format that can be re-imported using admin/mm/import.'),
    'aliases' => array('mmx'),
    'arguments' => array(
      'mmtid' => 'The Tree ID of the page to start at',
    ),
    'required-arguments' => TRUE,
    'options' => array(
      'nodes' => 'Include nodes',
    ),
  );
  $items['mm-find-unmodified-homepages'] = array(
    'description' => t('List user homepages that can probably be deleted.'),
    'aliases' => array('mm-unmodified', 'mmfuh'),
  );
  $items['mm-fix-orphan-nodes'] = array(
    'description' => t('Check for nodes not assigned to any MM page and associate them with a page.'),
    'aliases' => array('mm-fix-orphans', 'mmfon'),
  );
  $items['mm-fix-sort'] = array(
    'description' => t('Check for and fix any problems in the way menu entries are sorted.'),
    'aliases' => array('mm-fix', 'mmfs'),
  );
  $items['mm-mark-vgroups'] = array(
    'description' => t('Mark all virtual groups as "dirty", so that they are regenerated during the next cron run.'),
    'aliases' => array('mm-mark', 'mmmv'),
  );
  $items['mm-purge-unmodified-homepages'] = array(
    'description' => t('Delete or recycle user homepages that have not changed.'),
    'aliases' => array('mmpuh'),
  );
  $items['mm-update-vgroups'] = array(
    'description' => t('Update the membership of any "dirty" virtual groups immediately, instead of during cron.'),
    'aliases' => array('mm-update', 'mmuv'),
  );
  $items['mm-regenerate-vgroups'] = array(
    'description' => t('Update the membership of all virtual groups immediately.'),
    'aliases' => array('mm-regen', 'mmrv'),
  );
  return $items;
}

function drush_monster_menus_mm_check_orphan_nodes() {
  module_load_include('inc', 'monster_menus', 'mm_admin');
  mm_admin_orphan_nodes(FALSE, 'drush_log');
}

function drush_monster_menus_mm_check_sort() {
  module_load_include('inc', 'monster_menus', 'mm_admin');
  mm_admin_validate_sort_index(FALSE, 'drush_log');
}

function drush_monster_menus_mm_create_homepage($name_uid) {
  if (!($account = is_numeric($name_uid) ? user_load((int)$name_uid) : user_load_by_name($name_uid))) {
    return drush_set_error(dt('Could not find the user @user', array('@user' => $name_uid)));
  }

  if (!$account->status) {
    return drush_set_error(dt('User @name (@uid) is disabled.', array('@name' => $account->name, '@uid' => $account->uid)));
  }

  if (!empty($account->user_mmtid) && ($exists = mm_content_get($account->user_mmtid, MM_GET_FLAGS))) {
    $perms = mm_content_user_can($exists->mmtid);
    if ($perms[MM_PERMS_IS_RECYCLED]) {
      // Restore a page that is in the recycle bin.
      mm_content_move_from_bin($account->user_mmtid);
      drush_log(dt('Restored @name (@uid) from recycle bin to @path.', array('@name' => $account->name, '@uid' => $account->uid, '@path' => url('mm/' . $account->user_mmtid))), 'success');
      return;
    }

    if (($parent = mm_content_get($exists->parent)) && $parent->name == MM_ENTRY_NAME_DISABLED_USER) {
      // Restore a homepage that is in the "disabled" tree?
      if (drush_prompt(dt('User @name (@uid) already has a disabled homepage at @path. Restore it or create a new one? (r/n)', array('@name' => $account->name, '@uid' => $account->uid, '@path' => url('mm/' . $account->user_mmtid)))) == 'r') {
        mm_content_move($account->user_mmtid, mm_content_users_mmtid());
        drush_log(dt('Restored @name (@uid) from the disabled section to @path.', array('@name' => $account->name, '@uid' => $account->uid, '@path' => url('mm/' . $account->user_mmtid))), 'success');
        return;
      }
      unset($exists->flags['user_home']);
      mm_content_set_flags($exists->mmtid, $exists->flags);
      $account->user_mmtid = NULL;
    }
    else {
      return drush_set_error(dt('User @name (@uid) already has a homepage at @path.', array('@name' => $account->name, '@uid' => $account->uid, '@path' => url('mm/' . $account->user_mmtid))));
    }
  }
  // Create from scratch.
  mm_content_add_user($account);
  drush_log(dt('Created a homepage for @name (@uid) at @path.', array('@name' => $account->name, '@uid' => $account->uid, '@path' => url('mm/' . $account->user_mmtid))), 'success');
}

function drush_monster_menus_mm_disable_homepage($name_uid) {
  if (!($account = is_numeric($name_uid) ? user_load((int) $name_uid) : user_load_by_name($name_uid))) {
    return drush_set_error(dt('Could not find the user @user', array('@user' => $name_uid)));
  }

  if (empty($account->user_mmtid) || !($exists = mm_content_get($account->user_mmtid))) {
    return drush_set_error(dt('User @name (@uid) does not have a valid homepage.', array('@name' => $account->name, '@uid' => $account->uid)));
  }

  if (($parent = mm_content_get($exists->parent)) && $parent->name == MM_ENTRY_NAME_DISABLED_USER) {
    return drush_set_error(dt('User @name (@uid) already has a disabled homepage at @path.', array('@name' => $account->name, '@uid' => $account->uid, '@path' => url('mm/' . $account->user_mmtid))));
  }

  if (!($err = mm_content_move_to_disabled($account->user_mmtid))) {
    drush_log(dt('Moved the homepage of @name (@uid) to @path.', array('@name' => $account->name, '@uid' => $account->uid, '@path' => url('mm/' . $account->user_mmtid))), 'ok');
    return;
  }
  drush_set_error($err);
}

function drush_monster_menus_mm_find_unmodified_homepages() {
  global $user;
  $user = user_load(1);

  function _mm_fuh_print($item) {
    drush_log($item->mmtid . ' ' . url('mm/' . $item->mmtid), 'ok');
  }
  mm_content_find_unmodified_homepages('_mm_fuh_print');
}

function drush_monster_menus_mm_purge_unmodified_homepages() {
  global $user;
  $user = user_load(1);

  function _mm_puh_purge($item) {
    static $count = 0;
    print dt("Purged: @count\r", array('@count' => ++$count));
    mm_content_recycle_enabled() ? mm_content_move_to_bin($item->mmtid) : mm_content_delete($item->mmtid);
  }
  drush_log(dt(mm_content_recycle_enabled() ? 'Moved @count homepage(s) to the recycle bin.' : 'Deleted @count homepage(s).', array('@count' => mm_content_find_unmodified_homepages('_mm_puh_purge'))), 'ok');
}

function drush_monster_menus_mm_fix_orphan_nodes() {
  module_load_include('inc', 'monster_menus', 'mm_admin');
  mm_admin_orphan_nodes(TRUE, 'drush_log');
}

function drush_monster_menus_mm_fix_sort() {
  module_load_include('inc', 'monster_menus', 'mm_admin');
  mm_admin_validate_sort_index(TRUE, 'drush_log');
  mm_content_update_sort_queue();
}

function drush_monster_menus_mm_regenerate_vgroups() {
  mm_content_update_vgroup_view();
  mm_regenerate_vgroup();
}

function drush_monster_menus_mm_mark_vgroups() {
  mm_content_update_vgroup_view();
}

function drush_monster_menus_mm_update_vgroups() {
  mm_regenerate_vgroup();
}

function drush_monster_menus_mm_dump($mmtid) {
  module_load_include('inc', 'monster_menus', 'mm_admin');
  mm_admin_dump($mmtid);
}

function drush_monster_menus_mm_export($mmtid) {
  global $user;
  module_load_include('inc', 'monster_menus', 'mm_import_export');
  $old_user = $user;
  drupal_save_session(FALSE);
  $user = user_load(1);
  print mm_export($mmtid, drush_get_option('nodes'));
  $user = $old_user;
  drupal_save_session(TRUE);
}
